%
% This script performs a fit of reconstructed sea levels from the
% Mann et al (2008) PNAS temperature reconstruction, for the past 
% 1.5 millennia, to sea level data for the salt marches of North Carolina, 
% Tump Point and Sand Point (Horton et al.)
% It performs a Bayesian update to Monte Carlo generated sea level curves
% using the formal gaussian uncertainty on the sea level points as 
% likelihoods.
%
% (c) 2009 Martin Vermeer
%

% Plotting parameters
ed = 15;    % for SSA smoothing used
bins = 20;  % for plotting parameter likelihood histograms
model = 'd'; % 'd' for PNAS recon & three-parm relationship, 
             % 'r' for original (Rahmstorf) relationship + tau
apriori = false;   % 'true' to plot prior info without bayesian updating
firstyear = 1000;  % first year for which to include sea level proxy data
%firstyear = 0;  % first year for which to include sea level proxy data
showfirstyear = 'n';
firstyear2 = 1100; % last year of MWP (medieval warmth period)

%MWPcorr = 0.2;    % correction for period before firstyear2 to temperatures; 
                   % is described in "An attempt at characterizing the early 
                   % period descrepancy"; used for supplementary figure S5
MWPcorr = 0.0;     % otherwise set MWPcorr = 0.0

bandfit = false;    % true to plot uncertainty band with raw SL data
fullband = false;  % forces non-cutaway view of band
recent = false;    % generate plot of instrumental period and timeseries
oldmodels = 'n';   % adds lines to plot that show projection of old models
loose = 'n';       % looser constraints on a, b and T0
loosefac = 10;
a_loose = 0.44;
b_loose = 0.0;
T0_loose = -0.45;

quickie = 'n';     % use much less samples in posterior comp, speeds up

GIA_offset = 0.0;  % +/- 0.015 cm/yr;
%GIA_offset = -0.015;

%Fingerprint = 0.83; % is worst case scenario, which means sea-level values 
                     % of North Carolina are assumed to be only 83% of the 
                     % global average (is described "Ice sheet gravity 
                     % effect on the proxy sea-level data"); used for
                     % supplementary figure S2;
                     % in  Mitrovica 2001 for NC
Fingerprint = 1.0;   % otherwise set Fingerprint = 1.0

mvnrnd_test = false;

if apriori        % no bayesian updating; 
  sample = 1000;  % only 1000 samples
  lim = 0.0;      % no likelihood restrictions
  tau = 400;      % relaxation time for the 'short' (a2) adjustment of 
                  % equilibrium sea level
else
  if quickie == 'y'
    sample = 1000;
  else
    if loose == 'y'
      sample = 250000;
    else
      sample = 25000;
    end
  end
  lim = 1.0e-8;     % cut-off for small likelihoods (10^(-8))
  pointlim = lim;   % used for plotting likelihoods
  tau = 400;        % relaxation time for the 'short' (a2) adjustment of 
                    % equilibrium sea level
end

fac1 = 1.0e1; % adjustment factor for autocorrelated sea level data as 
              % described in "Bayesian updating to estimate the model parameters"
H0 = 2.0;    % Integration constant for sea level, adjusts reconstructed
             % sea level to the level of the North Carolina data
a1 = 0.07;   % secular response part

%---
% GIA correction, cm/yr. Positive means subsidence
NC_GIA = -0.0 + GIA_offset; % NC above and beyond the assumed values 1 and 0.9 mm/yr
NC_GIA2 = 0.095; % value used in skewing error boxes
WI_GIA =  0.020; % Scituate Wood Island
P_GIA  =  0.010; % Pierce Island

fprintf(1, '\ntau = %8.0f\n', tau);

startyear = 500;
load pnas.txt
% PNAS reconstruction (Mann et al. (2008))
year = pnas(:,1);     % yeardates derived from proxy data 
obs  = pnas(:,2);     % temperatures derived from proxy data
if model == 'd'
  % Dual model params; see equation (3) and (4) in paper
  a = 0.56;   
  a2 = a - a1; 
  b = -4.9;   
  % How much equilibrium temp was under ref level for "anomalies"
  % Note that anomalies are relative to 1951-1980. 
  % For the 20th century obs fit, we obtained T0 = -0.41, with 
  % an uncertainty around +/- 0.03 (Jackknife fit),  
  T0_instr = -0.41; 
  extra_offset = 0.0;
else
  % Simple model with Chao corr
  a = 0.44; a2 = a - a1;
  b = 0.0;
  T0_instr = -0.45;
  extra_offset = 0.0;
end

nyrs = size(year, 1); % number of years from PNAS reconstruction

% Mean value for / correction to 1400-1800:
Trefcorr = mean(obs(1400-startyear+1:1800-startyear+1)); % mean for 1400-1800 for reconstructed temperatures by Mann et al.
fprintf(1, '\nT level for 1400-1800: %8.5f\n', Trefcorr);
%Hrefcorr = -25.1905; % In cm, ref. Andy Kemp 13.01.2010
%Hrefcorr = -23.68; % In cm, own estimate (median)
Hrefcorr = -23.92; % In cm, own estimate (poly)
fprintf(1, '\nH level for 1400-1800: %8.5f\n', Hrefcorr);
 
% The above value of T0_instr was derived for a ref period of 1951-1980
Toffset = mean(obs(1951-startyear+1:1980-startyear+1)); %  mean for 1951-1980 for reconstructed temperatures by Mann et al.
T0_instr = T0_instr + Toffset + extra_offset; % shifting T0_instr to same ref period as reconstructed temperatures
fprintf(1, '\nT0 fixed part assumed: %5.3f\n', T0_instr);

%====compute sea level projections for the older models of Rahmstorf 2007 and Vermeer&Rahmstorf 2009
if oldmodels == 'y'
  % (This is only done to plot them for comparison purposes in figure 1)
  a_ = 0.56; T0_ = -0.41; b_ =-4.9; % values from Vermeer and Rahmstorf 2009
  T0_ = -.35; % adjustment of T0 to fit proxy data better
  [H9 Ht9]=seaproject(a_,b_,T0_,obs-Toffset);
  H9=H9-mean(H9(1400-startyear+1:1800-startyear+1)); % average for 1400-1800 is used as zero level

  a_ = 0.34; T0_ = -0.5; b_ = 0; % values from Rahmstorf (2007)
  T0_ = -.35; % adjustment of T0 to fit proxy data better
  [H7 Ht7]=seaproject(a_,b_,T0_,obs-Toffset);
  H7=H7-mean(H7(1400-startyear+1:1800-startyear+1)); % average for 1400-1800 is used as zero level
end

%====proxy data====
% sand point data; columns: 1:Age, 2:Relative Seal Level (RSL), 3: Age Error, 4:RSL Error
sand = xlsread('SP and TP final RSL datasets.xls', 'Sand Point'); 
% tump point data; columns: 1:Age, 2:Relative Seal Level (RSL), 3: Age Error, 4:RSL Error
tump = xlsread('SP and TP final RSL datasets.xls', 'Tump Point'); 
Syear=sand(:,1); % sand point yeardates [yrs]
Ssea=sand(:,2).*100 ;  %sand point Relative Sea Level [cm]
SF=sand(:,3);    %sand point age error [yrs]
SE=sand(:,4).*100;    %sand point RSL error [cm]

Tyear=tump(:,1); % tump point yeardates [yrs]
Tsea=tump(:,2).*100 ;  %tump point Relative Sea Level [cm]
TF=tump(:,3);    %tump point age error[yrs]
TE=tump(:,4).*100;    %tump point RSl error [cm]

%compute "detrended" data
Ssea = (Ssea + (0.1 + GIA_offset)*(2007 - Syear)) / Fingerprint; % for sand point: 1 mm/yr subsidence
Tsea = (Tsea + (0.09 + GIA_offset)*(2007 - Tyear)) / Fingerprint; % for tump point: 0.9 mm/yr subsidence


fprintf('\nData points:\nSand Point  %6d\nTump Point  %6d\n\n', size(Syear,1), size(Tyear,1));


%=====Assumed prior statistics; See Table S2=====
% Temperature
r1 = randn(round(nyrs/10 + 1), sample); % One random value for every 10 years
% here randn generates  a round(nyrs/10 + 1)-by-sample matrix of random 
% numbers whose elements are normally distributed with mean 0, variance 1

% tau
r2 = 4.0 * (rand(1, sample) - 0.5); % here rand generates an array of random 
% numbers whose elements are uniformly distributed in the interval (-2,2).

% a1
r3 = 0.5 * rand(1, sample) - 0.06; % here rand generates an array of random 
% numbers whose elements are uniformly distributed in the interval (-0.06,0.44)


% H0
if loose
  r5 = 20 * (rand(1, sample) - 0.5);
else
  r5 = 10 * (rand(1, sample) - 0.5); % here rand generates an array of random 
% numbers whose elements are uniformly distributed in the interval (-5,5).
end

% for T0_instr
% here randn generates  a 1-by-sample matrix of random 
% numbers whose elements are normally distributed with mean 0, 
% standard deviation as given 0.03 or 0.15
if loose == 'y'
  r4 = loosefac * 0.03 * randn(1, sample);
  loosefac
else
  r4 = 0.03 * randn(1, sample);
end
% For T0(1)
r6 = 0.2 * randn(1, sample);% here randn generates  a 1-by-sample matrix of random 
% numbers whose elements are normally distributed with mean 0, standard deviation 0.2
% For a
% here randn generates  a 1-by-sample matrix of random 
% numbers whose elements are normally distributed with mean 0, 
% standard deviation 0.05 or 0.25
if loose == 'y'
  r7 = loosefac * 0.05 * randn(1, sample);
else
  r7 = 0.05 * randn(1, sample);
end

if loose == 'y'
  r8 = 0.0 * randn(1, sample); % b; we cannot estimate separately so constrain
else
  r8 = 1.0 * randn(1, sample); % b
end

i2 = 1;
fprintf(1, 'Sample loop \nTried  Used\n');
for i=1:sample
  if mod(i,sample/100) == 0
    fprintf(1, '%5d %5d\r', i, i2);
  end
  w(i2) = 1.0;
  tau_rnd = tau * exp(r2(1,i)); % tau = 400
  a1_rnd  = a1 + r3(1,i);       % a1 = 0.07; r3 is U(-0.06,0.44); therefore: a1_rnd is U(0.01,0.51)
  if loose == 'y'
    b_rnd = b_loose + r8(1,i);
    a_rnd = a_loose + r7(1,i);     % use R07 but with Chao
    T0_instr_rnd = T0_loose + r4(1,i);
  else
    b_rnd = b + r8(1,i);
    a_rnd = a + r7(1,i); % a = 0.56; r7 is N(0,0.05^2); therefore: a_rnd is N(0.56,0.05^2)
    T0_instr_rnd = T0_instr + r4(1,i); %T0_instr = T0_instr + Toffset + extra_offset = -0.41 - 0.11 + 0.0
  end
  a2_rnd = a_rnd - a1_rnd;      % equation (3)
  H0_rnd = H0 + r5(1,i);        %H0 = 2.0
  temp(1) = obs(1) + 0.15 * r1(1,i); %obs is temperature from Mann et al.(2008)
  for j = 2:nyrs
    temp(j) = obs(j) + 0.15 * r1(round(j/10+1),i); %distribution is N(0,0.15^2K)for T(t<AD1850); see Table S2
  end

  % For real instrumental data, use conservative values according to
  % P. Brohan, J.J. Kennedy, I. Harris, S.F.B. Tett & P.D. Jones (2006):
  % Uncertainty estimates in regional and global observed
  % temperature changes: a new dataset from 1850; 
  % J. Geophysical Research 111, D12106, doi:10.1029/2005JD006548
  % http://www.cru.uea.ac.uk/cru/data/temperature/HadCRUT3_accepted.pdf
  % Figure 10c
  for j = 1950-startyear+1:nyrs
    temp(j) = obs(j) + 0.04 * r1(round(j/10+1),i); %distribution is N(0,0.04^2K)for T(AD 1950-2006); see Table S2
  end
  for j = 1850-startyear+1:1950-startyear
    temp(j) = obs(j) + 0.06 * r1(round(j/10+1),i); %distribution is N(0,0.06^2K)for T(AD 1850-1950); see Table S2
  end

% EXPERIMENTAL LOWERING of Medieval Warm Period (MWP)
  for j = 1:nyrs
    if year(j) < firstyear2         % only until 1100
      temp(j) = temp(j) - MWPcorr;  % MWPcorr = 0.0
    end
  end

  % Initialize by first 200 years (AD 500-700)
  T0(1) = mean(temp(1:200)) + r6(1,i) + MWPcorr; % r6 is N(0,0.2^2); therefore T0(1) is N(mean(AD500-700),0.02^2)
  for j = 2:nyrs
    % Compute here T0(j) as the tau-year memory of temp(j)
    T0(j) = T0(j-1) + (1/tau_rnd)*(temp(j) - T0(j-1));    % equation (2b)
  end

  % Move effective T0 to T0_instr level over 1880-2000 period:
  DT = mean(T0(1880-startyear+1:2000-startyear+1)) - T0_instr_rnd;
  T0_0_rnd = T0_instr_rnd - a2_rnd * DT / a1_rnd;
  % explanation: equation (3) in the paper says t0 = (a1*t0_0 + a2*<t0>) /a
  %                                        <==> t0_0 = a/a1*t0 - a2/a1*<t0>
  % here t0 is named T0_instr_rnd; with T0_instr_rnd = T0_instr + Toffset + r4(1,i)
  % (see equation (4) in the paper and line 115 for Toffset);
  % DT = <t0> - t0 = mean(T0(1880-2000)) - T0_instr_rnd;
  % we therefore yield (by using a = a1 + a2):  
  % t0_0_rnd = (a*t0 - a2*<t0>)/a1 = t0 + a2/a1*t0 - a2/a1*<t0>
  %          = t0 - a2/a1*(<t0> - t0) = T0_instr_rnd - (a2/a1)*DT 
  
  %fprintf('%d %8.4f %8.4f | %8.4f %8.4f | %8.4f %8.4f\n', i, T0(1), T0(1400), DT, T0_0_rnd, a1_rnd, a2_rnd);

  stemp = ssatrend([year temp'], ed); % Finds the ssa non-linear trend from a time series; Rahmstorf et al. 2007; Moore et al. 1995
  dtemp = ([0 diff(stemp)']);
  dsea = a1_rnd*(stemp' - T0_0_rnd) + a2_rnd*(stemp' - T0) + b_rnd*dtemp; % equation (2a); dH/dt
  sea = cumsum(dsea);   % seal level at times of year(i)
  sea = sea - sea(2000-startyear+1) + H0_rnd; % setting sea level to H0_rnd at year 2000 (H0_rnd = 2 cm + U(-5,5)cm)
  
if ~apriori  % bayesian updating
  for k = 1:size(Syear,1)
    for j = 1:nyrs
      if year(j) == round(Syear(k)) && year(j) > firstyear
        sq = (Ssea(k) - sea(j))^2;
        w(i2) = w(i2) * exp(-0.5*sq/(fac1*SE(k)^2)); % Likelihood function 
                    % as described in equation (5) in the paper
                    % here computing for sand point data
      end
    end
  end
  for k = 1:size(Tyear,1)
    for j = 1:nyrs
      if year(j) == round(Tyear(k)) && year(j) > firstyear
        sq = (Tsea(k) - sea(j))^2;
        w(i2) = w(i2) * exp(-0.5*sq/(fac1*TE(k)^2));  % Likelihood function 
                    % as described in equation (5) in the paper
                    % here computing for tump point data    
      end
    end
  end
end
  % keep only the somewhat likely ensemble members
  if w(i2) > lim
    %fprintf(1, '%5f\r', i2);
    % Collect in table:
    st(i2,:) = stemp;   % smoothed temperature
    ss(i2,:) = sea;     % sea level  computed with (2a)
    % "Effective" or "total" T0 to be plotted:
    tt(i2,:) = (a1_rnd * T0_0_rnd + a2_rnd * T0(:)) / (a1_rnd + a2_rnd); % equation (3); T0
    ttp(i2,:) = T0(:); % "Partial" T0
    tr(i2) = tau_rnd;   % tau
    a1r(i2) = a1_rnd;    % a1
    a2r(i2) = a2_rnd;   % a2
    t0r(i2) = T0_0_rnd; % T0_0
    H0r(i2) = H0_rnd;   % H0
    br(i2) = b_rnd;     % b
    i2 = i2 + 1; % index for counting up
  end
end

% make size equal to others (st etc)
w = w(1:i2 - 1);

fprintf(1, '%5d samples kept out of %5d tried\n', i2 - 1, sample);
sample = i2 - 1;

% Scale to unit probability
w = w / sum(w);

% Computation of percentiles
% sea level
for j = 1:nyrs
  [sss,ix] = sort(ss(:,j)); % sorts sea level vector at time j such that sss(1) >= ... >=sss(length(sss))
  cw = cumsum(w(ix));       % computes cumulated sums of the likeslihoods analogically to how the ss vecotr was sorted
  for m = 1:5
    P(m,j) = sss(1);
  end
  for i = 2:sample % go through all the entries in cw
    L = [0.0228 0.1587 0.5 0.8413 0.9772]; % values for one and 2 sigma bounds for normal distribution
    for m = 1:5
      if cw(i-1) < L(m) && cw(i) >= L(m) % and find L(m) that lies between cw(i-1) and cw(i); 
        P(m,j) = sss(i); % the smallest value of sss(i) is saved in row m for which 
        % cw(i-1) < L(m) && cw(i) >= L(m) is true, which means with probability L(m) sss(1),..,sss(i) are above the value sss(i)
      end
    end
  end
end

%======================
% ===Start plotting====
%======================

% Sea level
figure(1); clf; hold on; box on;
%axes('Position',[0 0 1 0.5]);
color1 = [.8 .8 .8];
color2 = [.7 .7 .7];

%set axis
xlabel('Year', 'FontSize', 12);
ylabel('Sea level (cm)', 'FontSize', 12);
axis([0 2200 -200 50])
if ~apriori && showfirstyear == 'y'
  plot([firstyear firstyear], [-200 50], 'g');
end
if bandfit && fullband
  axis([-300 2100 -70 20])
end
if recent
  axis([1850 2000 -5 35])
end


% plot SL predicted from Mann et al. temperature (median with 1 and 2sigma error bands) ; shifting to 1400-1800 (Hrefcorr)
sy = 1;
if ~bandfit 
  xs=[year(sy:nyrs); flipud(year(sy:nyrs))]; % sy=1
  ys=[P(1,sy:nyrs)'; flipud(P(5,sy:nyrs)')] - Hrefcorr; % two sigma bound
  fill(xs,ys,color1,'edgecolor','none');
  ys=[P(2,sy:nyrs)'; flipud(P(4,sy:nyrs)')] - Hrefcorr; % one simga bound
  fill(xs,ys,color2,'edgecolor','none');
  median_SL = P(3,:) - Hrefcorr;                           % median
  fprintf(1, '\nMann et al. ave. for 1400-1800: %8.5f\n', mean(median_SL(1400-startyear:1800-startyear))); % test
end

fid = fopen('sol.txt','w');
for i = 1:nyrs
  % median and +/- one sigma output
  fprintf(fid, '%8d %8.5f %8.5f %8.5f\n', year(i), P(2,i)-Hrefcorr, P(3,i)-Hrefcorr, P(4,i)-Hrefcorr);
end
fclose(fid);

% Plot poly fit to Sand and Tump + error band
[byear,ix] = sort([Tyear' Syear']);
bsea = [Tsea' Ssea']';
bsea2 = bsea(ix);
[p,s,mu] = polyfit(byear', bsea2, 9); % finds coefficients p of polynomial of degree 9 that fits the data p(x) to bsea2 in least square sense; 
% with x = (byear - mu )/s; mu = mean(byear), s = std(byear)
[sbsea err] = polyval(p,byear',s,mu); % evaluates p at (byear - mu )/s; err gives estimate of standard deviations: sbsea +/- err contains 50% of predictions

[sea1418 err2] = polyval(p,[1400:1800]',s,mu); 
fprintf(1, '\nPoly ave. for 1400-1800: %8.5f\n', mean(sea1418) - Hrefcorr); % test


if ~recent
  if bandfit || fullband % plot uncertainty band of 9 degree polynomial fitted to NC SL data
    %span = [1:106];
    span = [1:size(byear,2)];
    xm = byear';
    xs = [xm(span); flipud(xm(span))];
    % We use a visually estimated uncertainty of +/- 0.05 m for the proxy
    % data points
    yh = sbsea+10; % +2sigma
    yl = sbsea-10; % -2sigma
    ys = [yh(span); flipud(yl(span))] - Hrefcorr;
    fill(xs,ys,[1. .9 .9],'edgecolor','none');
    yh = sbsea+5;   % one sigma
    yl = sbsea-5;   % -one sigma 
    ys = [yh(span); flipud(yl(span))] - Hrefcorr;
    fill(xs,ys,[1. .8 .8],'edgecolor','none');
  else
    for i = 1:3 % different cutaways
      if i==1
        span = [20:35];
      end
      if i==2
        span = [50:80];
      end
      if i==3
        span = [100:130];
      end
      xm = byear';
      xs = [xm(span); flipud(xm(span))];
      yh = sbsea+10; % two
      yl = sbsea-10; % sigma
      ys = [yh(span); flipud(yl(span))] - Hrefcorr;
      fill(xs,ys,[1. .9 .9],'edgecolor','none');
      yh = sbsea+5; % convert to 
      yl = sbsea-5; % one sigma
      ys = [yh(span); flipud(yl(span))] - Hrefcorr;
      fill(xs,ys,[1. .8 .8],'edgecolor','none');
    end
  end
end

% plot median of SL predicted from Mann et al temperature
if ~bandfit
  plot(year, median_SL, '-k'); 
end

if bandfit
  % Plot error parallellograms of raw SL data; figure 1C
  for i = 1:size(Syear,1)
    x = [Syear(i)+SF(i), Syear(i)-SF(i),Syear(i)-SF(i),Syear(i)+SF(i),Syear(i)+SF(i)];
    y = [Ssea(i)+SE(i)-NC_GIA2*SF(i),Ssea(i)+SE(i)+NC_GIA2*SF(i),Ssea(i)-SE(i)+NC_GIA2*SF(i),Ssea(i)-SE(i)-NC_GIA2*SF(i),Ssea(i)+SE(i)-NC_GIA2*SF(i)];
    plot(x,y - Hrefcorr,'g');
  end
  for i = 1:size(Tyear,1)
    x = [Tyear(i)+TF(i), Tyear(i)-TF(i),Tyear(i)-TF(i),Tyear(i)+TF(i),Tyear(i)+TF(i)];
    y = [Tsea(i)+TE(i)-NC_GIA2*TF(i),Tsea(i)+TE(i)+NC_GIA2*TF(i),Tsea(i)-TE(i)+NC_GIA2*TF(i),Tsea(i)-TE(i)-NC_GIA2*TF(i),Tsea(i)+TE(i)-NC_GIA2*TF(i)];
    plot(x,y - Hrefcorr,'c');
  end
end

% 9 degree polynomial fitted to NC SL data
if ~recent 
  plot(byear, sbsea - Hrefcorr, '-r');
end

% add the projections from the older models (smoothed with ssatrend)
if oldmodels == 'y'
  plot(year,ssatrend([year H7],15),'g');
  plot(year,ssatrend([year H9],15),'b');
end

% plot of supplementary figure 6; Church an White 
if recent
  load church_13221.txt;
  seayear = (church_13221(:,1));
  sealevel = church_13221(:,2)/10-12.5 + 1.65 + (3.7/3.1415)*atan2(seayear-1978,13); % Apply Chao et al. (2008) reservoir correction
  %%sealevel = church_13221(:,2)/10-10;
  sealevel2 = ssatrend([seayear sealevel], ed);
  plot(seayear, sealevel - Hrefcorr,'-b');
end

% Temperature
figure(2); clf; hold on; box on;
%axes('Position', [0 0.5 1 1]);
axis([0 2200 -0.5 1])

% Re-center on instrumental period, i.e., estimate and remove the small shift
% introduced by the SSA smoothing:
trobs = ssatrend([year obs], ed);
m = mean(trobs(1880-startyear+1:2000-startyear+1));
for i = 1:sample
  mm = mean(st(i,1880-startyear+1:2000-startyear+1));
  st(i,:) = st(i,:) - mm + m;
end

% Computation of percentiles
% temperature ;(explanation: see computaion of percentiles Sea level)
for j = 1:nyrs
  fprintf(1, '#');
  [sst,ix] = sort(st(:,j));
  cw = cumsum(w(ix));
  for m = 1:5
    P(m,j) = sst(1);
  end
  for i = 2:sample
    L = [0.0228 0.1587 0.5 0.8413 0.9772];
    for m = 1:5
      if cw(i-1) < L(m) && cw(i) >= L(m)
        P(m,j) = sst(i);
      end
    end
  end
end

% plot 1 and 2 sigma error bands of reconstructed temperature
xs=[year; flipud(year)];
ys=[P(1,:)'; flipud(P(5,:)')] - Trefcorr;
fill(xs,ys,[.8 .8 .8],'edgecolor','none');
ys=[P(2,:)'; flipud(P(4,:)')] - Trefcorr;
fill(xs,ys,[.7 .7 .7],'edgecolor','none');

median_T = P(3,:) - Trefcorr; % median of temperature

% Computation percentiles 
% T0 ;(explanation: see computation of percentiles Sea level)
for j = 1:nyrs
  fprintf(1, '#');
  [ttt,ix] = sort(tt(:,j));
  cw = cumsum(w(ix));
  for m = 1:5
    P(m,j) = ttt(1);
  end
  for i = 2:sample
    L = [0.0228 0.1587 0.5 0.8413 0.9772];
    for m = 1:5
      if cw(i-1) < L(m) && cw(i) >= L(m)
        P(m,j) = ttt(i);
      end
    end
  end
end

% plot median and 1 and 2sigma error bands of T0; figure 2A and 3A:
% "Equlibrium Temperature"; shifting to 1400-1800 (Trefcorr)
median_T0 =  P(3,:) - Trefcorr;
fprintf('\nMedian t0eff at 1900: %8.4f\n', P(3, 1900-startyear+1) - Trefcorr);
ys=[P(1,:)'; flipud(P(5,:)')] - Trefcorr;
fill(xs,ys,[.9 1 .9],'edgecolor','none');
ys=[P(2,:)'; flipud(P(4,:)')] - Trefcorr;
fill(xs,ys,[.8 1 .8],'edgecolor','none');
plot(year,median_T0, '-g');

% plot median of temperature reconstruction (Mann et al.) (figure 2A and 3A)
plot(year, median_T, '-k');
if ~apriori
  plot(year, trobs - Trefcorr, '-b'); % figure 3A: "Original timeseries temperature"
end

xlabel('Year', 'FontSize', 12);
%ylabel('Temps relative 1961-1990 (K)', 'FontSize', 12);
ylabel('Temps relative 1400-1800 (K)', 'FontSize', 12);
axis 'auto y'

% Distributions of parameters  
if ~apriori % only if bayesian update was done
  figure(3); clf; hold on; box on;

  % H0
  [s,ix] = sort(H0r(:));
  ww = w(ix);
  y(1:bins+1) = 0.0;
  range = s(size(s,1)) - s(1);
  for i = 1:size(s,1)
    j = floor((s(i)-s(1)) * bins / range) + 1;
    y(j) = y(j) + ww(i);
  end
  x = [0:bins] * range / bins + s(1);
  subplot(3,3,1);
  fill([x(1) x x(bins+1)], [0 y 0], [.7 .7 .7]);
  xlabel('H_0 (cm)');
  ylabel('Likelihood');

  % a1
  [s,ix] = sort(a1r(:));
  ww = w(ix);
  y(1:bins+1) = 0.0;
  range = s(size(s,1)) - s(1);
  for i = 1:size(s,1)
    j = floor((s(i)-s(1)) * bins / range) + 1;
    y(j) = y(j) + ww(i);
  end
  x = [0:bins] * range / bins + s(1);
  subplot(3,3,2);
  fill([x(1) x x(bins+1)], [0 y 0], [.7 .7 .7]);
  xlabel('a_1 (cm a^{-1} K^{-1})');

  % tau
  [s,ix] = sort(tr(:));
  ww = w(ix);
  y(1:bins+1) = 0.0;
  range = s(size(s,1)) - s(1);
  for i = 1:size(s,1)
    j = floor((s(i)-s(1)) * bins / range) + 1;
    y(j) = y(j) + ww(i);
  end
  x = [0:bins] * range / bins + s(1);
  subplot(3,3,4);
  fill([x(1) x x(bins+1)], [0 y 0], [.7 .7 .7]);
  xlabel('\tau (a)');
  ylabel('Likelihood');

  % T_0,0
  [s,ix] = sort(t0r(:) - Trefcorr); % shifting to 
  ww = w(ix);
  y(1:bins+1) = 0.0;
  range = s(size(s,1)) - s(1);
  for i = 1:size(s,1)
    j = floor((s(i)-s(1)) * bins / range) + 1;
    y(j) = y(j) + ww(i);
  end
  x = [0:bins] * range / bins + s(1);
  subplot(3,3,5);
  fill([x(1) x x(bins+1)], [0 y 0], [.7 .7 .7]);
  xlabel('T_{0,0} (K)');

  % a1*T_0,0
  [s,ix] = sort(-a1r(:).*(t0r(:) - Trefcorr));
  ww = w(ix);
  y(1:bins+1) = 0.0;
  range = s(size(s,1)) - s(1);
  for i = 1:size(s,1)
    j = floor((s(i)-s(1)) * bins / range) + 1;
    y(j) = y(j) + ww(i);
  end
  x = [0:bins] * range / bins + s(1);
  subplot(3,3,6);
  fill([x(1) x x(bins+1)], [0 y 0], [.7 .7 .7]);
  xlabel('-a_1 T_{0,0} (cm a^{-1})');

  likely = (w > pointlim);
  fprintf(1, 'Plotted %d points out of %d.\n', size(tr(likely),2), size(tr,2));

  subplot(3,3,7);
  plot(tr(likely), a1r(likely), '.', 'markersize', 1, 'color', [0 0.5 0]);
  xlabel('\tau (a)');
  ylabel('a_1 (cm a^{-1} K^{-1})');

  subplot(3,3,8);
  plot(t0r(likely) - Trefcorr, a1r(likely), '.', 'markersize', 1, 'color', [0 0.5 0]);
  xlabel('T_{0,0} (K)');

  subplot(3,3,9);
  plot(-a1r(likely).*(t0r(likely) - Trefcorr), a1r(likely), '.', 'markersize', 1, 'color', [0 0.5 0]);
  %axis([0 0.4 0 0.5])
  xlabel('-a_1 T_{0,0} (cm a^{-1})');

end

% means and variances
%%if ~apriori
  % These are the values for which mean and covariance matrix will be formed.
  % Some, like tau and T00 are not used directly as they are very nonlinear
  val(1,:) = H0r(:);
  %val(2,:) = (ttp(:,nyrs)-Trefcorr); % effective T0(2000AD)
  val(2,:) = ttp(:,1); % "Partial" T0(500AD)
  val(3,:) = log(tr(:)); % tau
  %val(3,:) = 1./(tr(:)) % tau
  val(4,:) = a1r(:); % a1
  val(5,:) = a1r(:)+a2r(:); % a
  %val(6,:) = a1r(:).*(t0r(:)-Trefcorr); % a1*T00
  val(6,:) = a1r(:).*t0r(:); % a1*T00
  val(7,:) = br(:); % b

  for i = 1:7
    mu(i) = sum(val(i,:).*w)/sum(w);
  end
  for i = 1:7
    for j = 1:7
      sig(i,j) = sum((val(i,:)-mu(i)).*(val(j,:)-mu(j)).*w)/sum(w);
    end
  end
  if sig(7,7) == 0.0
    sig(7,7) = 1.0; % variance of b according to PNAS paper
  end

  %fprintf('\nMeans:\n\nH0         %10.6f\nT0(2000AD) %10.6f\nln(tau)    %10.6f\na1         %10.6f\na          %10.6f\na1*T00     %10.6f\nb          %10.6f\n\n', mu);
  fprintf('\nMeans:\n\nH0         %10.6f\nT0(500AD)  %10.6f\nln(tau)    %10.6f\na1         %10.6f\na          %10.6f\na1*T00     %10.6f\nb          %10.6f\n\n', mu);
  %fprintf('\nMeans:\n\nH0         %10.6f\nT0(500AD)  %10.6f\n1/tau    %10.6f\na1         %10.6f\na          %10.6f\na1*T00     %10.6f\nb          %10.6f\n\n', mu);
  fprintf('\nCovariance matrix\n');
  fprintf('\n %10.6f %10.6f %10.6f %10.6f %10.6f %10.6f %10.6f', sig);

  e = eig(sig);
  fprintf('\n\nEigenvalues\n');
  fprintf('\n %8.4f %8.4f %8.4f %8.4f %8.4f %8.4f %8.4f', e);
  fprintf('\n\n');
%MV
if ~apriori
%MVend

  for i = 1:7
    for j = 1:7
      sig2(i,j) = sig(i,j)/(sqrt(sig(i,i))*sqrt(sig(j,j)));
    end
  end
  fprintf('\nCorrelation matrix\n');
  fprintf('\n %8.4f %8.4f %8.4f %8.4f %8.4f %8.4f %8.4f', sig2);
  e = eig(sig2);
  fprintf('\n\nEigenvalues\n');
  fprintf('\n %8.4f %8.4f %8.4f %8.4f %8.4f %8.4f %8.4f', e);
  fprintf('\n\n');

  figure(4); clf; hold on; box on;
  plot(year, median_SL, 'k');
  plot(byear, sbsea - Hrefcorr, 'r');

  figure(5); clf; hold on; box on;
  plot(year, stemp - Trefcorr, 'r');
  plot(year, median_T, 'k');
  plot(year, median_T0, 'k');

  nk = 20;
  for k = 1:nk
  if mvnrnd_test
    SLRpara_values=mvnrnd(mu,sig,nk)';
    H0 = SLRpara_values(1,k);
    T0(1) = SLRpara_values(2,k);
    %tau = exp(SLRpara_values(3,k));
    tau = 1/SLRpara_values(3,k);
    a1 = SLRpara_values(4,k);
    a = SLRpara_values(5,k);
    a2 = a - a1;
    T0_0 = SLRpara_values(6,k)/a1;
    b = SLRpara_values(7,k);
    %fprintf('\nSample:\n\nH0         %10.6f\nT0(500AD)  %10.6f\nln(tau)    %10.6f\na1         %10.6f\na          %10.6f\na1*T00     %10.6f\nb          %10.6f\n\n', SLRpara_values(:,k));
    fprintf('\n We plot one mvnrnd realization based on (mu,sig)');
    fprintf('\nThis replaces the blue mean curve in Figs 4,5\n');
  else
    % Take the parameter means:
    H0 = mu(1);
    T0(1) = mu(2);
    tau = exp(mu(3));
    %tau = 1/mu(3);
    a1 = mu(4);
    a = mu(5);
    a2 = a - a1;
    T0_0 = mu(6)/a1;
    b = mu(7);
  end
  
  fprintf('Tau: %8.4f\n', tau);

  % Now we should work in the 'non-Trefcorrected' temperature origin:
  stemp = ssatrend([year obs], ed); 
  dtemp = ([0 diff(stemp)']);
  for j = 2:nyrs
    T0(j) = T0(j-1) + (1/tau)*(stemp(j) - T0(j-1));
  end
  dsea = a1*(stemp' - T0_0) + a2*(stemp' - T0) + b*dtemp; 
  sea_av = cumsum(dsea);
  sea_av = sea_av - sea_av(2000-startyear+1) + H0 - Hrefcorr; 
  % It is the "total" T0(t) we want to plot:
  T0_av = (a1*T0_0 + a2*T0) / a - Trefcorr;

  sea_sample(:,k) = sea_av(:);
  T0_sample(:,k) = T0_av(:);
  end; % k

  % Take the most likely sample member:
  lh = 0.0; j = 0;
  for i = 1:size(w,2)
    if w(i) > lh
      lh = w(i);
      j = i;
    end
  end

  H0 = val(1,j);
  T0(1) = val(2,j);  
  tau = exp(val(3,j));
  %tau = 1/val(3,j);
  a1 = val(4,j);
  a = val(5,j);
  a2 = a - a1;
  T0_0 = val(6,j)/a1;
  b = val(7,j);

  fprintf('\nML values:\n\nH0         %8.4f\nT0(500AD)  %8.4f\nln(tau)    %8.4f\na1         %8.4f\na          %8.4f\na1*T00     %8.4f\nb          %8.4f\n\n', val(:,j));
  %fprintf('\nML values:\n\nH0         %8.4f\nT0(500AD)  %8.4f\n1/tau    %8.4f\na1         %8.4f\na          %8.4f\na1*T00     %8.4f\nb          %8.4f\n\n', val(:,j));
  fprintf('Tau: %8.4f\n', tau);
  
  for j = 2:nyrs
    T0(j) = T0(j-1) + (1/tau)*(stemp(j) - T0(j-1));
  end
  dsea = a1*(stemp' - T0_0) + a2*(stemp' - T0) + b*dtemp; 
  sea_ml = cumsum(dsea);
  sea_ml = sea_ml - sea_ml(2000-startyear+1) + H0 - Hrefcorr; 
  % "Partial" to "total" again for plotting:
  T0_ml = (a1*T0_0 + a2*T0) / a - Trefcorr;

  figure(4);
  if mvnrnd_test
    for k = 1:nk
      plot(year, sea_sample(:,k), ':b');
    end
  else
    plot(year, sea_av, 'b');
  end
  plot(year, sea_ml, 'g');

  figure(5);
  if mvnrnd_test
    for k = 1:nk
      plot(year, T0_sample(:,k), ':b');
    end
  else
    plot(year, T0_av, '--b');
  end
  plot(year, T0_ml, '--g');
end

